/*
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Id: espda_frame.h 161769 2014-05-30 14:32:34Z gcupani $
 * $Author: gcupani $
 * $Date: 2014-05-30 16:32:34 +0200 (Fri, 30 May 2014) $
 * $Revision: 161769 $
 */

#ifndef ESPDA_FRAME_H
#define ESPDA_FRAME_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_frame_name(A, B)  \
            char A[LEN_STRING]; \
            strcpy(A, B);       \
            strcat(A, ".fits"); \

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "espda_dfs.h"
#include "espda_check.h"
#include "espda_spec.h"
#include "espda_util.h"

/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_size        espda_frame_adjust (cpl_frameset             *,
    	                            const char               *,
    	                            cpl_size);

cpl_error_code  espda_frame_check  (cpl_frameset             *,
                                    cpl_recipeconfig         *,
                                    const char	             *);

cpl_error_code  espda_frame_desc   (cpl_frameset   	     *,
                                    const char    	     *,
                                    const char      	     *,
                                    ...);

cpl_error_code  espda_frame_desc_old(cpl_frame 	   	     *,
                                    char      	   	     *,
                                    double    	   	     *);

cpl_error_code  espda_frame_export (cpl_frameset             *,
                                    const cpl_parameterlist  *,
                                    cpl_table                *,
                                    cpl_propertylist         *,
                                    const char    	     *,
                                    const char    	     *,
                                    const char    	     *);

cpl_table      *espda_frame_collect(cpl_frameset             *,
                                    cpl_recipeconfig         *,
                                    const char               *,
                                    cpl_size,
				    int,
                                    cpl_array                *,
                                    cpl_array               **,
                                    cpl_array               **,
                                    cpl_array               **);

cpl_error_code  espda_frame_extract_old(cpl_frameset             *,
                                    double,
                                    double,
                                    cpl_table               **,
                                    cpl_table               **,
                                    cpl_array               **,
                                    cpl_array               **);

cpl_error_code  espda_frame_import (cpl_frameset             *,
                                    char                     *,
                                    cpl_table               **);

cpl_table      *espda_frame_load   (cpl_frameset             *,
                                    cpl_recipeconfig         *,
                                    const char               *,
                                    cpl_size);

cpl_error_code  espda_frame_load_old(cpl_frameset             *,
                                    char                     *,
                                    cpl_frameset            **);


cpl_imagelist  *espda_image_load   (cpl_frameset 	     *,
                                    cpl_type         	      ,
                                    cpl_recipeconfig 	     *,
                                    char             	     *,
                                    int               	      );

cpl_imagelist *espda_get_image(cpl_frameset *,
                               cpl_type      ,
                               char          *);

cpl_error_code espda_frame_extract_by_tag(cpl_frameset 		*,
                                          const char		*,
                                          cpl_frameset		*);

cpl_table *espda_get_table(cpl_frameset *, char *);

cpl_size       espda_frame_size    (cpl_frameset             *);

#endif
