/*
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Id: espda_model.h 161538 2014-05-22 13:56:10Z gcupani $
 * $Author: gcupani $
 * $Date: 2014-05-22 15:56:10 +0200 (Thu, 22 May 2014) $
 * $Revision: 161538 $
 */

#ifndef ESPDA_UTIL_MODEL_H
#define ESPDA_UTIL_MODEL_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_model_chisq(A, B, C, D, E, F, G)                       \
            if (A != NULL) {                                         \
                cpl_table_new_column(A, "TEMP",    CPL_TYPE_DOUBLE); \
                cpl_table_new_column(A, "TEMPERR", CPL_TYPE_DOUBLE); \
                cpl_table_copy_data_double(A, "TEMP",                \
                    cpl_table_get_data_double(A, B));                \
                cpl_table_copy_data_double(A, "TEMPERR",             \
                    cpl_table_get_data_double(A, C));                \
                cpl_table_subtract_columns(A, "TEMP", D);            \
                cpl_table_multiply_columns(A, "TEMP", "TEMP");       \
                cpl_table_divide_columns(A, "TEMP", "TEMPERR");      \
                cpl_table_divide_columns(A, "TEMP", "TEMPERR");      \
                F = cpl_table_get_column_mean(A, "TEMP") *           \
                    cpl_table_get_nrow(A);                           \
                G = F / E;                                           \
                cpl_table_erase_column(A, "TEMP");                   \
                cpl_table_erase_column(A, "TEMPERR");                \
            }                                                        \


/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_deriv.h>
#include <gsl/gsl_multifit_nlin.h>
#include <math.h>

#include "espda_dfs.h"
#include "espda_interp.h"
#include "espda_spec.h"
#include "espda_voigt.h"

/*-----------------------------------------------------------------------------
                                   Structures
 -----------------------------------------------------------------------------*/

//######
struct gsl_multifit_data{cpl_table  *spec_over;   // Over-chunks         
                         cpl_table  *spec_chunk;  // Chunks         
                         cpl_table  *lfit_chunk;  // Line list         
                         cpl_table  *prof_chunk;  // Instrument profile        
                         cpl_table  *voigt_func;  // Voigt function
                         cpl_array  *par_fix;	  // Fixed parameters
                         cpl_array  *par_pos;	  // Positions of line params.
                         cpl_table  *par_lim;     // Limits    on line params. 
                         cpl_size    par_num;	  // Number    of line params.
                         cpl_array  *ion_wavel;	  // Transition wavelength
                         cpl_array  *ion_oscs;	  // Transition osc. strength
                         cpl_array  *ion_damp;	  // Transition damping
                         };	          
/*
struct gsl_multifit_data{cpl_table  *spec_over;   // Over-chunks         
                         double  *spec_chunk;  // Chunks
  cpl_array *spec_norm;
  cpl_array *spec_normerr;
                         cpl_table  *lfit_chunk;  // Line list         
                         cpl_table  *prof_chunk;  // Instrument profile        
                         cpl_table  *voigt_func;  // Voigt function
                         cpl_array  *par_fix;	  // Fixed parameters
                         cpl_array  *par_pos;	  // Positions of line params.
                         cpl_table  *par_lim;     // Limits    on line params. 
                         cpl_size    par_num;	  // Number    of line params.
                         cpl_array  *ion_wavel;	  // Transition wavelength
                         cpl_array  *ion_oscs;	  // Transition osc. strength
                         cpl_array  *ion_damp;	  // Transition damping
                         };	          
*/
//######

struct model_voigtf_params{const char *px;
                           double wavel_i;
                           double ion_wavel;
                           double ion_oscs;
    		  	   double ion_damp;
    		  	   double linez;
    		 	   double colden;
    		 	   double therb; 
    		 	   double turbb;
                           cpl_table *voigt_func;
                           };
                                 
/*-----------------------------------------------------------------------------
                                       Prototypes
 -----------------------------------------------------------------------------*/

double 	       espda_model_colden(const double,
       	                          const double);

double 	       espda_model_colden2(const char  *,
       	                          const double);
                                  
int    	       espda_model_costf  (const gsl_vector  *,
       	                           void              *,
       	                           gsl_vector        *); 
int    	       espda_model_costdf (const gsl_vector  *,
       	                           void              *,
       	                           gsl_matrix        *); 

/*
int    	       espda_model_costdf (CBLAS_TRANSPOSE_t  , 
                                   const gsl_vector  *,
                                   const gsl_vector  *, 
                                   void              *, 
                                   gsl_vector        *,
				   gsl_matrix        *);
*/ 

cpl_table     *espda_model_inst  (cpl_size,
                                  double,
                                  double);

cpl_array     *espda_model_transm (const gsl_vector  *,
       	                           void              *,
				   cpl_array        **,
				   gsl_matrix       **); 

int    	       espda_model_transm_old(const gsl_vector  *,
       	                           void              *,
       	                           gsl_vector        *); 

double         espda_model_voigt  (double,
				   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   cpl_table         *);
/*
double         espda_model_voigtf (double,
				   void              *);
*/
cpl_error_code espda_model_voigt_old  (cpl_array         *,
                                   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   double,
                                   cpl_table         *,
                                   cpl_array        **,
                                   cpl_array        **,
                                   double            *);  


#endif
