/* $Id: espda_synth_ccf.h 2018-09-08 14:15:10 jonaydas $
 *
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: jonaydas $
 * $Date: 2018-09-08 14:15:10 +0000 (sat, 02 may 2018) $
 * $Revision: 198015 $
 * $Name:  $
 */

#ifndef ESPDA_SYNTH_CCF_H
#define ESPDA_SYNTH_CCF_H

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>

#include <espda_fit_gauss.h>

/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_error_code espda_ccf_fit_synth(cpl_array     *wobs,
			           cpl_array     *fobs,
			           cpl_array     *fsyn,
			           cpl_array     *wmsk,
			           cpl_array     *wcor,
			           cpl_array     *fres,
			           cpl_array    **xccf,
			           cpl_array    **yccf,
			           cpl_array     *pccf,
			           cpl_array     *eccf,
			           cpl_array    *pccfc,
			           cpl_array    *eccfc,
				       cpl_array   **ygfit,
				       cpl_array **ygfcore,
                       double        rvccf,
                       double        vwccf,
                       int        fitg,
                       int       frv);

cpl_error_code espda_ccf_fit_gauss(cpl_array     *xarr,
 			           cpl_array     *yarr,
 			           cpl_array     *yfit,
 			           cpl_array     *bfit,
 			           cpl_array     *pfit,
 			           cpl_array     *efit,
				       double         ymin,
					   double         rvi);

cpl_error_code espda_ccf_fit_lorntz(cpl_array     *xarr,
 			           cpl_array     *yarr,
 			           cpl_array     *yfit,
 			           cpl_array     *bfit,
 			           cpl_array     *pfit,
 			           cpl_array     *efit,
				       double         ymin,
				       double         rvi);

cpl_error_code espda_ccf_fit_par(cpl_array     *xarr,
 			           cpl_array     *yarr,
 			           cpl_array     *yfit,
 			           cpl_array     *pfit,
 			           cpl_array     *efit,
				       double         rvi);

void gaussfit(double t[], double y[], double sig[], int npt,
              double gcf[], double gcf_er[], int npar, int *stat2);

void lorntzfit(double t[], double y[], double sig[], int npt,
               double gcf[], double gcf_er[], int npar, int *stat2);

void parfit(double t[], double y[], double sig[], int npt,
               double gcf[], double gcf_er[], int npar, int *stat2);

//void print_state (int iter, gsl_multifit_fdfsolver * s);

#endif
